/*
**                TUnsharp v0.9 for AviSynth 2.5.x
**
**   TUnsharp is a sharpening filter that uses a few different variations
**   of unsharpmasking, and allows for controlled sharpening based on edge
**   magnitude and min/max neighborhood value clipping.  It also sports a
**   a gui with real time preview.
**   
**   Copyright (C) 2004 Kevin Stone
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <vfw.h>
#include "internal.h"
#include "resource.h"

// some global variables for gui
HWND hDlgg;
HWND hWndg;
HDRAWDIB hddg;
static HINSTANCE hInstanceg;
HANDLE hThreadg;
DWORD idg;
BITMAPINFOHEADER bi;
int gstrength;
int gthresholdL;
int gthresholdU;
int gtype;
int gmap;
int glim;
int trackg;
// end global variables

class TUnsharp : public GenericVideoFilter
{
	int strength, thresholdL, thresholdU;
	int type, nfrms, lim, map, radius, track;
	int strengths, thresholdLs, thresholdUs, types, lims, maps;
	bool gui;
	PClip blurred;
	PVideoFrame storeDst, storeOrg, storeSrc, tdst;
	void TUnsharp::copyFrame(PVideoFrame &dst, PVideoFrame &src, IScriptEnvironment *env);
	void TUnsharp::prepareMap(PVideoFrame &dst);
	void TUnsharp::copyFrameForBMP(PVideoFrame &src);
	bool TUnsharp::checkMatching();
	void TUnsharp::copyFrameNoEnv(PVideoFrame &dst, PVideoFrame &src);

public:
	int itrack;
	bool TUnsharp::redoFrame(bool force);
	void TUnsharp::grabBMPFrame();
	void TUnsharp::grabBMPFrameLimited();
	bool TUnsharp::restoreDefaults();
	TUnsharp::~TUnsharp();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

    TUnsharp(PClip _child, int _strength, int _thresholdL, int _thresholdU, int _type, int _map, int _lim,
		int _radius, bool _gui, PClip _blurred, IScriptEnvironment* env) : GenericVideoFilter(_child), 
		strength(_strength), thresholdL(_thresholdL), thresholdU(_thresholdU), type(_type), map(_map), lim(_lim), 
		radius(_radius), gui(_gui), blurred(_blurred)
	{
		if (!(child->GetVideoInfo().IsYV12()) && !(child->GetVideoInfo().IsYUY2()))
			env->ThrowError("TUnsharp:  YV12 and YUY2 data only!");
		if (type < 0 || type > 5)
			env->ThrowError("TUnsharp:  type must be set to 0, 1, 2, 3, 4, or 5!");
		if (thresholdL < 0 || thresholdL > 256)
			env->ThrowError("TUnsharp:  thresholdL must be at least 0 and less then 257!");
		if (thresholdU < 0 || thresholdU > 256)
			env->ThrowError("TUnsharp:  thresholdU must be at least 0 and less then 257!");
		if (thresholdU < thresholdL)
			env->ThrowError("TUnsharp:  thresholdU must be greater then or equal to thresholdL!");
		if (strength < 0 || strength > 512)
			env->ThrowError("TUnsharp:  strength must be at least 0 and less then 513!");
		if (lim < 0 || lim > 256)
			env->ThrowError("TUnsharp:  lim must be at least 0 and less then 257!");
		if (map < 0 || map > 2)
			env->ThrowError("TUnsharp:  map must be set to 0, 1, or 2!");
		if (gui)
		{
			storeOrg = env->NewVideoFrame(child->GetVideoInfo());
			storeSrc = env->NewVideoFrame(blurred->GetVideoInfo());
			storeDst = env->NewVideoFrame(child->GetVideoInfo());
			tdst = env->NewVideoFrame(child->GetVideoInfo());
			memset(&bi, 0, sizeof(bi));
			bi.biSize = sizeof(bi);
			bi.biWidth = vi.width;
			bi.biHeight = vi.height;
			bi.biPlanes = 1;
			bi.biBitCount = vi.BitsPerPixel();
			if (vi.IsYUY2()) bi.biCompression = '2YUY';
			else bi.biCompression = '21VY';
			bi.biSizeImage = vi.BMPSize();
			trackg = 0;
			strengths = strength;
			thresholdLs = thresholdL;
			thresholdUs = thresholdU;
			types = type;
			lims = lim;
			maps = map;
		}
		track = -1;
		itrack = 0;
		nfrms = vi.num_frames-1;
		thresholdL *= thresholdL*4;
		thresholdU *= thresholdU*4;
	}
};

bool TUnsharp::restoreDefaults()
{
	int count = 0;
	if (gmap != maps) { map = gmap = maps; ++count; }
	if (gstrength != strengths) { strength = gstrength = strengths; ++count; }
	if (gthresholdL*gthresholdL*4 != thresholdLs*thresholdLs*4) 
	{
		++count;
		gthresholdL = thresholdLs; 
		thresholdL = thresholdLs*thresholdLs*4;
	}
	if (gthresholdU*gthresholdU*4 != thresholdUs*thresholdUs*4) 
	{
		++count;
		gthresholdU = thresholdUs;
		thresholdU = thresholdUs*thresholdUs*4;
	}
	if (glim != lims) { lim = glim = lims; ++count; }
	if (gtype != types) { type = gtype = types; ++count; }
	if (count != 0) 
	{
		track -= 1;
		return false;
	}
	return true;
}

bool TUnsharp::checkMatching()
{
	if (!gui || !hDlgg) return true;
	int count = 0;
	if (gmap != map) { map = gmap; ++count; }
	if (gstrength != strength) { strength = gstrength; ++count; }
	if (gthresholdL*gthresholdL*4 != thresholdL) { thresholdL = gthresholdL*gthresholdL*4; ++count; }
	if (gthresholdU*gthresholdU*4 != thresholdU) { thresholdU = gthresholdU*gthresholdU*4; ++count; }
	if (glim != lim) { lim = glim; ++count; }
	if (gtype != type) { type = gtype; ++count; }
	if (count != 0) return false;
	return true;
}

void TUnsharp::grabBMPFrame()
{
	if (trackg != track)
	{
		trackg = track;
		void *lpBuffer2 = (void*)storeDst->GetReadPtr();
		HDC hDC = GetDC(hWndg);
		DrawDibDraw(hddg,hDC,0,0,bi.biWidth,bi.biHeight,&bi,lpBuffer2,0,0,bi.biWidth,bi.biHeight,NULL);
		ReleaseDC(hWndg, hDC);  // don't leak, or bad things happen ;)
	}
	else grabBMPFrameLimited();
}

void TUnsharp::grabBMPFrameLimited()
{
	if (trackg == track)
	{
		void *lpBuffer2 = (void*)storeDst->GetReadPtr();
		PAINTSTRUCT ps;
		HDC hDC = BeginPaint(hWndg, &ps);
		DrawDibDraw(hddg,hDC,0,0,bi.biWidth,bi.biHeight,&bi,lpBuffer2,0,0,bi.biWidth,bi.biHeight,NULL);
		EndPaint(hWndg, &ps);  // don't leak
	}
	else grabBMPFrame();
}

PVideoFrame __stdcall TUnsharp::GetFrame(int n, IScriptEnvironment *env)
{
	if (n < 0) n = 0;
	else if (n > nfrms) n = nfrms;
	checkMatching();
	PVideoFrame src = blurred->GetFrame(n, env);  // this is the line that causes an error
	                                              // when not compiled in debug mode
	PVideoFrame org = child->GetFrame(n, env);
	PVideoFrame dst = env->NewVideoFrame(child->GetVideoInfo());
	if (map>1) prepareMap(dst);
	else copyFrame(dst, org, env);
	const unsigned char *srcp = src->GetReadPtr(PLANAR_Y);
	int src_pitch = src->GetPitch(PLANAR_Y);
	srcp += src_pitch;
	const unsigned char *srcpp = srcp - src_pitch;
	const unsigned char *srcpn = srcp + src_pitch;
	const unsigned char *orgp = org->GetReadPtr(PLANAR_Y);
	int width = org->GetRowSize(PLANAR_Y);
	int height = org->GetHeight(PLANAR_Y);
	int org_pitch = org->GetPitch(PLANAR_Y);
	orgp += org_pitch;
	const unsigned char *orgpp = orgp - org_pitch;
	const unsigned char *orgpn = orgp + org_pitch;
	unsigned char *dstp = dst->GetWritePtr(PLANAR_Y);
	int dst_pitch = dst->GetPitch(PLANAR_Y);
	dstp += dst_pitch;
	int x, y, v, Ix2, Iy2, maxL, minL;
	float c1, c2, vf;
	int step = vi.IsYV12() ? 1 : 2;
	int startx = step;
	if (map>0)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU) dstp[x] = 255;
			}
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 0)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = (srcp[x]<<2)-srcpp[x]-srcp[x-step]-srcp[x+step]-srcpn[x];
					v = orgp[x] + ((v*strength)>>6);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 1)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = (srcp[x]<<3)-srcpp[x-step]-srcpp[x]-srcpp[x+step]-
						             srcp[x-step]-srcp[x+step]-
									 srcpn[x-step]-srcpn[x]-srcpn[x+step];
					v = orgp[x] + ((v*strength)>>7);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 2)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = ((srcp[x]*srcp[x])<<1)-srcpn[x-step]*srcpp[x+step]-srcpp[x-step]*srcpn[x+step];
					v = orgp[x] + ((v*strength)>>14);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 3)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = Ix2*((srcp[x]<<1)-srcp[x-step]-srcp[x+step]) + Iy2*((srcp[x]<<1)-srcpp[x]-srcpn[x]);
					v = orgp[x] + ((v*strength)>>15);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 4)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					c1 = Ix2 / (Ix2*Ix2*0.001f+250.0f);
					c2 = Iy2 / (Iy2*Iy2*0.001f+250.0f);
					vf = c1*((srcp[x]<<1)-srcp[x-step]-srcp[x+step]) + c2*((srcp[x]<<1)-srcpp[x]-srcpn[x]);
					v = orgp[x] + (int)(vf*strength*0.0625f);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 5)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = orgp[x] - srcp[x];
					v = orgp[x] + ((v*strength)>>7);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	if (gui && hDlgg)
	{
		track = n;
		copyFrameForBMP(dst);
		copyFrame(storeSrc, src, env);
		copyFrame(storeOrg, org, env);
	}
	return dst;
}

bool TUnsharp::redoFrame(bool force)
{
	if (!force && checkMatching()) return false;
	if (map>1) prepareMap(tdst);
	else copyFrameNoEnv(tdst, storeOrg);
	const unsigned char *srcp = storeSrc->GetReadPtr(PLANAR_Y);
	int src_pitch = storeSrc->GetPitch(PLANAR_Y);
	srcp += src_pitch;
	const unsigned char *srcpp = srcp - src_pitch;
	const unsigned char *srcpn = srcp + src_pitch;
	const unsigned char *orgp = storeOrg->GetReadPtr(PLANAR_Y);
	int width = storeOrg->GetRowSize(PLANAR_Y);
	int height = storeOrg->GetHeight(PLANAR_Y);
	int org_pitch = storeOrg->GetPitch(PLANAR_Y);
	orgp += org_pitch;
	const unsigned char *orgpp = orgp - org_pitch;
	const unsigned char *orgpn = orgp + org_pitch;
	unsigned char *dstp = tdst->GetWritePtr(PLANAR_Y);
	int dst_pitch = tdst->GetPitch(PLANAR_Y);
	dstp += dst_pitch;
	int x, y, v, Ix2, Iy2, maxL, minL;
	float c1, c2, vf;
	int step = vi.IsYV12() ? 1 : 2;
	int startx = step;
	if (map>0)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU) dstp[x] = 255;
			}
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 0)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = (srcp[x]<<2)-srcpp[x]-srcp[x-step]-srcp[x+step]-srcpn[x];
					v = orgp[x] + ((v*strength)>>6);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 1)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = (srcp[x]<<3)-srcpp[x-step]-srcpp[x]-srcpp[x+step]-
						             srcp[x-step]-srcp[x+step]-
									 srcpn[x-step]-srcpn[x]-srcpn[x+step];
					v = orgp[x] + ((v*strength)>>7);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 2)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = ((srcp[x]*srcp[x])<<1)-srcpn[x-step]*srcpp[x+step]-srcpp[x-step]*srcpn[x+step];
					v = orgp[x] + ((v*strength)>>14);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 3)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = Ix2*((srcp[x]<<1)-srcp[x-step]-srcp[x+step]) + Iy2*((srcp[x]<<1)-srcpp[x]-srcpn[x]);
					v = orgp[x] + ((v*strength)>>15);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 4)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					c1 = Ix2 / (Ix2*Ix2*0.001f+250.0f);
					c2 = Iy2 / (Iy2*Iy2*0.001f+250.0f);
					vf = c1*((srcp[x]<<1)-srcp[x-step]-srcp[x+step]) + c2*((srcp[x]<<1)-srcpp[x]-srcpn[x]);
					v = orgp[x] + (int)(vf*strength*0.0625f);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	else if (type == 5)
	{
		for (y=1; y<height-1; ++y)
		{
			for (x=startx; x<width-1; x+=step)
			{
				Ix2 = srcp[x+step]-srcp[x-step]; 
				Ix2 *= Ix2;
				Iy2 = srcpp[x]-srcpn[x]; 
				Iy2 *= Iy2;
				if (Ix2+Iy2 >= thresholdL && Ix2+Iy2 <= thresholdU)
				{
					v = orgp[x] - srcp[x];
					v = orgp[x] + ((v*strength)>>7);
					maxL = min(max(max(max(max(orgpp[x-step],orgpp[x]),
						               max(orgpp[x+step],orgp[x-step])),
									   max(max(orgp[x],orgp[x+step]),
									   max(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) + lim, 255);
					minL = max(min(min(min(min(orgpp[x-step],orgpp[x]),
						               min(orgpp[x+step],orgp[x-step])),
									   min(min(orgp[x],orgp[x+step]),
									   min(orgpn[x-step],orgpn[x]))),
									   orgpn[x+step]) - lim, 0);
					if (v>maxL) v = maxL;
					else if (v<minL) v = minL;
					dstp[x] = v;
				}
			}
			orgpp += org_pitch;
			orgp += org_pitch;
			orgpn += org_pitch;
			srcpp += src_pitch;
			srcp += src_pitch;
			srcpn += src_pitch;
			dstp += dst_pitch;
		}
	}
	track -= 1;
	copyFrameForBMP(tdst);
	return true;
}

void TUnsharp::copyFrame(PVideoFrame &dst, PVideoFrame &src, IScriptEnvironment *env)
{
	int b, plane, np = vi.IsYV12() ? 3 : 1;
	for (b=0; b<np; ++b)
	{
		if (b == 0) plane = PLANAR_Y;
		else if (b == 1) plane = PLANAR_V;
		else plane = PLANAR_U;
		env->BitBlt(dst->GetWritePtr(plane), dst->GetPitch(plane), src->GetReadPtr(plane),
			src->GetPitch(plane), src->GetRowSize(plane), src->GetHeight(plane));
	}
}

void TUnsharp::copyFrameNoEnv(PVideoFrame &dst, PVideoFrame &src)
{
	int b, plane, dst_pitch, src_pitch, width, height, y, np = vi.IsYV12() ? 3 : 1;
	unsigned char *dstp;
	const unsigned char *srcp;
	for (b=0; b<np; ++b)
	{
		if (b == 0) plane = PLANAR_Y;
		else if (b == 1) plane = PLANAR_V;
		else plane = PLANAR_U;
		dstp = dst->GetWritePtr(plane);
		dst_pitch = dst->GetPitch(plane);
		srcp = src->GetReadPtr(plane);
		src_pitch = src->GetPitch(plane);
		width = src->GetRowSize(plane);
		height = src->GetHeight(plane);
		for (y=0; y<height; ++y)
		{
			memcpy(dstp, srcp, width);
			dstp += dst_pitch;
			srcp += src_pitch;
		}
	}
}

void TUnsharp::copyFrameForBMP(PVideoFrame &src)
{
	unsigned char *dstp = storeDst->GetWritePtr(PLANAR_Y);
	const unsigned char *srcp = src->GetReadPtr(PLANAR_Y);
	int pitchY = src->GetPitch(PLANAR_Y);
	int widthY = src->GetRowSize(PLANAR_Y);
	int heightY = src->GetHeight(PLANAR_Y);
	int out_pitch = (widthY+3) & -4; // word aligned
	int y;
	for (y=0; y<heightY; ++y)
	{
		memcpy(dstp, srcp, widthY);
		dstp += out_pitch;
		srcp += pitchY;
	}
	if (vi.IsYUY2()) return;
	int widthUV = src->GetRowSize(PLANAR_V);
	int heightUV = src->GetHeight(PLANAR_V);
	int pitchUV = src->GetPitch(PLANAR_V);
	srcp = src->GetReadPtr(PLANAR_V);
	for (y=0; y<heightUV; ++y)
	{
		memcpy(dstp, srcp, widthUV);
		dstp += out_pitch>>1;
		srcp += pitchUV;
	}
	srcp = src->GetReadPtr(PLANAR_U);
	for (y=0; y<heightUV; ++y)
	{
		memcpy(dstp, srcp, widthUV);
		dstp += out_pitch>>1;
		srcp += pitchUV;
	}
}

void TUnsharp::prepareMap(PVideoFrame &dst)
{
	if (vi.IsYV12())
	{
		memset(dst->GetWritePtr(PLANAR_Y),0,dst->GetHeight(PLANAR_Y)*dst->GetPitch(PLANAR_Y));
		memset(dst->GetWritePtr(PLANAR_U),128,dst->GetHeight(PLANAR_U)*dst->GetPitch(PLANAR_U));
		memset(dst->GetWritePtr(PLANAR_V),128,dst->GetHeight(PLANAR_V)*dst->GetPitch(PLANAR_V));
	}
	else
	{
		unsigned char *dstp = dst->GetWritePtr();
		int height = dst->GetHeight();
		int pitch = dst->GetPitch();
		int width = dst->GetRowSize()>>1;
		int x, y;
		for (y=0; y<height; ++y)
		{
			for (x=0; x<width; ++x)
			{
				((unsigned short *)dstp)[x] = 0x8000;
			}
			dstp += pitch;
		}
	}
}

// Another global variable for the gui.
// This is a pointer to the instance of TUnsharp that has a gui.
// It is used so calls can be made to TUnsharp functions
// between avisynth getframe() calls allowing for a real time preview.
// There can only be 1 instance with a gui so there are no multiple
// instance worries...
TUnsharp *tMyFilter;
// end extra global variable

BOOL CALLBACK DialogProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hWnd;
	switch (uMsg) 
	{
		case WM_INITDIALOG:
		{
			hWnd = GetDlgItem(hdlg, IDC_STRENGTH);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 512));
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, gstrength);
			SetDlgItemInt(hdlg, IDC_ESTRENGTH, gstrength, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_THRESHOLDL);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 256));
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, gthresholdL);
			SetDlgItemInt(hdlg, IDC_ETHRESHOLDL, gthresholdL, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_THRESHOLDU);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 256));
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, gthresholdU);
			SetDlgItemInt(hdlg, IDC_ETHRESHOLDU, gthresholdU, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_LIM);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 256));
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, glim);
			SetDlgItemInt(hdlg, IDC_ELIM, glim, TRUE);
			hWnd = GetDlgItem(hdlg, IDC_TYPE);
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"0 - Linear (4)");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"1 - Linear (8)");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"2 - Teager");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"3 - Cubic");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"4 - Rational");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"5 - Subtract");
			SendMessage(hWnd, CB_SETCURSEL, (WPARAM)gtype, (LPARAM)FALSE);
			hWnd = GetDlgItem(hdlg, IDC_MAP);
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"0 - No Map");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"1 - Normal Map");
			SendMessage(hWnd, CB_ADDSTRING, (WPARAM)FALSE, (LPARAM)"2 - Binary Map");
			SendMessage(hWnd, CB_SETCURSEL, (WPARAM)gmap, (LPARAM)FALSE);
			return TRUE;
		}
		case WM_COMMAND:
		{
			switch(wParam)
			{
				case ID_OK:
				{
					DestroyWindow(hWndg);
					hWndg = NULL;
					EndDialog(hDlgg, 1);
					hDlgg = NULL;
					return TRUE;
				}
				case ID_PREVIEW:
				{
					ShowWindow(hWndg, IsWindowVisible(hWndg) ? SW_HIDE : SW_SHOW);
					return TRUE;
				}
				case ID_DEFAULTS:
				{
					if (!tMyFilter->restoreDefaults())
					{
						if (IsWindowVisible(hWndg)) 
						{
							tMyFilter->redoFrame(true);
							tMyFilter->grabBMPFrame();
						}
						hWnd = GetDlgItem(hDlgg, IDC_STRENGTH);
						SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, gstrength);
						SetDlgItemInt(hDlgg, IDC_ESTRENGTH, gstrength, TRUE);
						hWnd = GetDlgItem(hDlgg, IDC_THRESHOLDL);
						SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, gthresholdL);
						SetDlgItemInt(hDlgg, IDC_ETHRESHOLDL, gthresholdL, TRUE);
						hWnd = GetDlgItem(hDlgg, IDC_THRESHOLDU);
						SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, gthresholdU);
						SetDlgItemInt(hDlgg, IDC_ETHRESHOLDU, gthresholdU, TRUE);
						hWnd = GetDlgItem(hDlgg, IDC_LIM);
						SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, glim);
						SetDlgItemInt(hDlgg, IDC_ELIM, glim, TRUE);
						hWnd = GetDlgItem(hDlgg, IDC_TYPE);
						SendMessage(hWnd, CB_SETCURSEL, (WPARAM)gtype, (LPARAM)FALSE);
						hWnd = GetDlgItem(hDlgg, IDC_MAP);
						SendMessage(hWnd, CB_SETCURSEL, (WPARAM)gmap, (LPARAM)FALSE);
					}
					return TRUE;
				}
				default:
				{
					switch (HIWORD(wParam)) 
					{
						case CBN_SELCHANGE:
						{
							switch(LOWORD(wParam))
							{
								case IDC_TYPE:
								{
									gtype = SendMessage((HWND)lParam, CB_GETCURSEL, (WPARAM)FALSE, (LPARAM)FALSE);
									if (IsWindowVisible(hWndg) && tMyFilter->redoFrame(false)) tMyFilter->grabBMPFrame();
									return TRUE;
								}
								case IDC_MAP:
								{
									gmap = SendMessage((HWND)lParam, CB_GETCURSEL, (WPARAM)FALSE, (LPARAM)FALSE);
									if (IsWindowVisible(hWndg) && tMyFilter->redoFrame(false)) tMyFilter->grabBMPFrame();
									return TRUE;
								}
							}
							break;
						}
					}
					break;
				}
				break;
			}
			break;
		}
		case WM_HSCROLL:
		{
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_STRENGTH))
			{
				gstrength = SendMessage(GetDlgItem(hdlg, IDC_STRENGTH), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_ESTRENGTH, gstrength, TRUE);
				if (IsWindowVisible(hWndg) && tMyFilter->redoFrame(false)) tMyFilter->grabBMPFrame();
				return TRUE;
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_THRESHOLDL))
			{
				gthresholdL = SendMessage(GetDlgItem(hdlg, IDC_THRESHOLDL), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_ETHRESHOLDL, gthresholdL, TRUE);
				if (IsWindowVisible(hWndg) && tMyFilter->redoFrame(false)) tMyFilter->grabBMPFrame();
				return TRUE;
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_THRESHOLDU))
			{
				gthresholdU = SendMessage(GetDlgItem(hdlg, IDC_THRESHOLDU), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_ETHRESHOLDU, gthresholdU, TRUE);
				if (IsWindowVisible(hWndg) && tMyFilter->redoFrame(false)) tMyFilter->grabBMPFrame();
				return TRUE;
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_LIM))
			{
				glim = SendMessage(GetDlgItem(hdlg, IDC_LIM), TBM_GETPOS, 0, 0);
				SetDlgItemInt(hdlg, IDC_ELIM, glim, TRUE);
				if (IsWindowVisible(hWndg) && tMyFilter->redoFrame(false)) tMyFilter->grabBMPFrame();
				return TRUE;
			}
			break;
		}
	}
	return false;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
		case WM_COMMAND:
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_ERASEBKGND:
		{
			if (hWnd == hWndg) return 1;  // don't flicker
			break;
		}
		case WM_PAINT:
		{
			if (hWnd == hWndg && IsWindowVisible(hWnd)) 
			{
				if (GetForegroundWindow() == hWndg) tMyFilter->grabBMPFrame();
				else tMyFilter->grabBMPFrameLimited();
			}
			break;
		}
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
    }
	return 0;
}

void startWindow() 
{
	MSG msg;
	HICON hIcon1;
	WNDCLASSEX wcex;
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstanceg;
	wcex.hIcon			= LoadIcon(NULL, MAKEINTRESOURCE(IDI_ICON1));
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= "TUnsharp";
	wcex.hIconSm		= LoadIcon(NULL, MAKEINTRESOURCE(IDI_ICON1));
	RegisterClassEx(&wcex);
	hDlgg = CreateDialog(hInstanceg,MAKEINTRESOURCE(IDD_DIALOG1),NULL,DialogProc);
	hWndg = CreateWindow("TUnsharp", "TUnsharp - Real Time Preview", WS_DLGFRAME,
			0, 0, bi.biWidth+GetSystemMetrics(SM_CXBORDER)*6, 
			bi.biHeight+GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CYBORDER)*6, 
			NULL, NULL, hInstanceg, NULL);
	hIcon1 = LoadIcon(hInstanceg, MAKEINTRESOURCE(IDI_ICON1));
	SendMessage(hDlgg, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)hIcon1);
	SendMessage(hDlgg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hIcon1);
	SendMessage(hWndg, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)hIcon1);
	SendMessage(hWndg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hIcon1);
	RECT r;
	GetWindowRect(hDlgg,&r);
	SetWindowPos(hWndg,NULL,r.right,r.top,0,0,SWP_NOZORDER|SWP_NOSIZE);
	UpdateWindow(hWndg);
	ShowWindow(hWndg, SW_HIDE);
    while (GetMessage(&msg, NULL, 0, 0))
	{
		if (NULL == hDlgg || !IsDialogMessage(hDlgg, &msg)) 
		{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
		}
	}
}

BOOL APIENTRY DllMain(HANDLE hModule, ULONG ulReason, LPVOID lpReserved) 
{
	switch(ulReason)
	{
		case DLL_PROCESS_ATTACH:
			hInstanceg = (HINSTANCE)hModule;
			hddg = DrawDibOpen();
			hThreadg = NULL;
			break;
		case DLL_PROCESS_DETACH:
			DrawDibClose(hddg);
			break;
	}
    return TRUE;
}

TUnsharp::~TUnsharp()
{
	if (itrack != 0) 
	{
		if (hThreadg)
		{
			if (hWndg)
			{
				ShowWindow(hWndg, SW_HIDE);
				DestroyWindow(hWndg);
			}
			if (hDlgg)
			{
				ShowWindow(hDlgg, SW_HIDE);
				EndDialog(hDlgg, 1);
			}
			SuspendThread(hThreadg);
			hThreadg = NULL;
		}
		tMyFilter = NULL;
	}
}

AVSValue __cdecl Create_TUnsharp(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	PClip v;
	int radius = args[7].IsInt() ? args[7].AsInt() : 2;
	if (radius != 1 && radius != 2 && radius != 3)
		env->ThrowError("TUnsharp:  radius must be set to 1, 2, or 3!");
	if (args[0].IsClip()) v = args[0].AsClip();
	else env->ThrowError("TUnsharp:  first arg must be a clip!");
	try 
	{
		AVSValue blurargs[2] = { v, 1.0 };
		v = env->Invoke("Blur",AVSValue(blurargs,2)).AsClip();
		if (radius>1)
		{
			AVSValue blurargs2[2] = { v, 1.0 };
			v = env->Invoke("Blur",AVSValue(blurargs2,2)).AsClip();
		}
		if (radius>2)
		{
			AVSValue blurargs3[2] = { v, 1.0 };
			v = env->Invoke("Blur",AVSValue(blurargs3,2)).AsClip();
		}
	}
	catch (IScriptEnvironment::NotFound)
	{
		env->ThrowError("TUnsharp:  error on invoke (not found)!");
	}
	if (args[8].IsBool() && args[8].AsBool() == true)
	{
		if(!hThreadg && !tMyFilter) 
		{
			gstrength = args[1].IsInt() ? args[1].AsInt() : 100;
			gthresholdL = args[2].IsInt() ? args[2].AsInt() : 2;
			gthresholdU = args[3].IsInt() ? args[3].AsInt() : 40;
			gtype = args[4].IsInt() ? args[4].AsInt() : 0;
			gmap = args[5].IsInt() ? args[5].AsInt() : 0;
			glim = args[6].IsInt() ? args[6].AsInt() : 2;
			tMyFilter = new TUnsharp(args[0].AsClip(),args[1].AsInt(100),args[2].AsInt(2),args[3].AsInt(40),
				args[4].AsInt(0),args[5].AsInt(0),args[6].AsInt(2),args[7].AsInt(2),args[8].AsBool(0),v,env);
			tMyFilter->itrack = 52341;
			hThreadg = CreateThread(NULL, 10000, (unsigned long (__stdcall *)(void *))startWindow, 0, 0 , &idg);
			return tMyFilter;
		}
		else env->ThrowError("TUnsharp:  only one instance of TUnsharp with a gui is supported!");
	}
	return new TUnsharp(args[0].AsClip(),args[1].AsInt(100),args[2].AsInt(2),args[3].AsInt(40),
		args[4].AsInt(0),args[5].AsInt(0),args[6].AsInt(2),args[7].AsInt(2),args[8].AsBool(0),v,env);
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) 
{
    env->AddFunction("TUnsharp", "c[strength]i[thresholdL]i[thresholdU]i[type]i[map]i[lim]i[radius]i[gui]b", Create_TUnsharp, 0);
	return 0;
}